unit QRBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  barcode,quickrpt, Qrctrls;


type tBeforePrintEvent = procedure(sender:tObject; var BCDATA : string) of Object;
type tAfterPrintEvent  = procedure(sender:tObject; BCDATA : string)     of Object;

type
  TQRBarcode = class(TQRImage)
  private
    fDpi           : integer;
    fBcdata        : string;
    fHeight,
    fFixedlen      : integer;
    fFillupChar    : char;
    fLeadingChars,
    fTrailingChars : string;
    CompBarcode    : tBarCode;
    fBeforePrint   : tBeforePrintEvent;
    fAfterPrint    : tAfterPrintEvent;
    fBarKind       : tBarCodeType;
    procedure makeproper;
    procedure setdpi(nval:integer);
    procedure setfixedlen(nVal:integer);
    procedure setTrailingChars(cVal:string);
    procedure setLeadingChars(cVal:string);
    procedure setfillupchar(c:Char);
    procedure setBcdata(cVal:string);
    procedure setHeight(nVal:integer);
    { Private declarations }
  protected
    { Protected declarations }
  public
    procedure print(x,y:integer);           override;
    procedure loaded;                       override;
    constructor create(aOwner:tcomponent);  override;
    { Public declarations }
  published
    { Published declarations }
    property BarcodeType   : tBarCodeType      read fBarKind       write fBarKind;
    property DPI           : integer           read fDpi           write setdpi   DEFAULT 300;
    property Barcode       : string            read fBcdata        write setbcdata;
    property TrailingChars : string            read fTrailingChars write setTrailingChars;
    property LeadingChars  : string            read fLeadingChars  write setLeadingChars;
    property FixedLen      : integer           read fFixedlen      write setfixedlen;
    property FillupChar    : char              read fFillupChar    write setfillupchar DEFAULT #48;
    property BarCodeHeight : integer           read fHeight        write setheight DEFAULT 100;
    property BeforePrint   : tBeforePrintEvent read fBeforePrint   write fBeforePrint;
    property AfterPrint    : tAfterPrintEvent  read fAfterPrint    write fAfterPrint;
  end;

procedure Register;

implementation


procedure tQrBarcode.setHeight(nVal:integer);
begin
  fHeight := nVal;
end;

procedure tQrBarcode.setbcdata(cVal:string);
begin
  fBcData := cval;
end;

procedure tQrBarcode.setFillupChar(c:char);
begin
  fFillupChar := c;
end;

procedure tQrBarcode.setleadingChars(cVal:string);
begin
  fLeadingChars := cval;
end;

procedure tQrBarcode.setTrailingChars(cVal:string);
begin
  fTrailingChars := cval;
end;

procedure tQrBarcode.setdpi(nval:integer);
begin
  fDpi := nVal;
end;

procedure tQrBarcode.setfixedlen(nVal:integer);
begin
  fFixedlen := nVal;
end;

procedure tQrBarcode.makeproper;
var cCode : string;
begin
  cCode := trim(fBcdata);
  if fFixedlen > 0 then begin
    while length(cCode) < fFixedlen do begin
      cCode := fFillupChar+cCode;
    end;
  end;
  cCode := fLeadingChars+cCode+fTrailingChars;
  CompBarcode.barcode := cCode;
end;


procedure tQrBarcode.print(x,y:integer);
var error : boolean;
    st : string;
    mysize : tQrPrintablesize;
begin
  mysize := size;

  CompBarcode := tbarcode.create(self);
  compbarcode.DPI           := fDpi;
  compBarcode.BarCodeheight := fHeight;
  CompBarcode.BarCode       := fBcData;
  Case fBarKind Of
    EAN_13   : CompBarCode.Bartype:=EAN_13;
    CODE_39  : CompBarcode.Bartype:=CODE_39;
    XCODE_39 : CompBarcode.Bartype:=XCODE_39;
  else
    CompBarcode.BarType := CODE_39
  End;
  if assigned(fBeforeprint) then fBeforePrint(self,fBcdata);
  makeproper;
  stretch := true;
  error:=False;
  Try
    CompBarcode.TestBarcode;
  except
    On EWrongNumber Do begin st:='Wrong number of digits'; error:=True; end;
    On EOnlyNumeric DO begin st:='Only numbers allowed!'; error:=True; end;
    On ECheckDigitFail Do begin st:='Check digit NOT correct'; error:=True; end;
    On ECharacterInValid Do Begin st:='Illegal character';error:=True; end;
  End;
  if error Then
    MessageDlg(st, mtError, [mbOk], 0)
  Else begin
    picture.bitmap := CompBarcode.getbarcode;
    inherited print(x,y);
  end;
  if assigned(fAfterprint) then fAfterPrint(self,CompBarcode.barcode);
  CompBarcode.destroy;
  picture := NIL;
  size := mysize;
end;

procedure tQrBarcode.loaded;
begin
  if fDpi     = 0 then fDpi     := 300;
  if fHeight  = 0 then fHeight  := 100;
end;


constructor tQrBarcode.create(aOwner:tcomponent);
begin
  inherited create(aOwner);
  fDpi        := 300;
  ffillupchar := #48;
  fHeight     := 100;
end;

procedure Register;
begin
  RegisterComponents('QReport', [TQRBarcode]);
end;

end.

